// (C) 2001-2018 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns / 1ns
module alt_e40_pma_cg3cekq #(
    parameter ENABLE_KR4  = 0 
)   (
    output [3:0]    rx_transfer_ready,
    input  [3:0]    rx_aib_reset,
    input  [3:0]    rx_pcs_reset,
    output [3:0]    tx_transfer_ready,
    input  [3:0]    tx_aib_reset,
    input  [3:0]    tx_pcs_reset,
    output  [3:0]   tx_dll_lock,
    input   [3:0]   tx_serial_clk0,
    input           rx_cdr_refclk0,
    output  [3:0]   tx_serial_data,
    input   [3:0]   rx_serial_data,
    output  [3:0]   rx_is_lockedtoref,
    input           rx_set_locktoref,
    input           rx_set_locktodata,
    input   [3:0]   tx_coreclkin,
    input   [3:0]   rx_coreclkin,
    output  [3:0]   tx_clkout,
    output  [3:0]   rx_clkout,
    output  [3:0]   rx_pma_iqtxrx_clkout,				  
    input   [263:0] tx_parallel_data,
    input   [3:0]   tx_fifo_wr_en,
    input   [3:0]   tx_err_ins,
    output  [263:0] rx_parallel_data,
    input   [3:0]   rx_bitslip,
    input   [3:0]   rx_enh_clr_errblk_count,
    output  [3:0]   rx_enh_blk_lock,
    input   [3:0]   rx_fifo_rd_en,
    input   [3:0]   tx_digitalreset,
    input   [3:0]   tx_analogreset,
    input   [3:0]   rx_digitalreset,
    input   [3:0]   rx_analogreset,
    output  [3:0]   tx_cal_busy,
    output  [3:0]   tx_analogreset_stat,
    output  [3:0]   tx_digitalreset_stat,
    output  [3:0]   rx_is_lockedtodata,
    output  [3:0]   rx_cal_busy,
    output  [3:0]   rx_analogreset_stat,
    output  [3:0]   rx_digitalreset_stat,
    output  [3:0]   rx_data_valid,
    output  [3:0]   tx_fifo_full,
    output  [3:0]   tx_fifo_empty,
    output  [3:0]   tx_fifo_pfull,
    output  [3:0]   tx_fifo_pempty,
    output  [3:0]   rx_fifo_full,
    output  [3:0]   rx_fifo_empty,
    output  [3:0]   rx_fifo_pfull,
    output  [3:0]   rx_fifo_pempty,
    input           reconfig_clk,
    input           reconfig_reset,
    input           reconfig_write,
    input           reconfig_read,
    input   [12:0]  reconfig_address,
    input   [31:0]  reconfig_writedata,
    output  [31:0]  reconfig_readdata,
    output          reconfig_waitrequest,
    input   [3:0]   rx_seriallpbken
);

    localparam NUM_CHANNELS = 4;
   // Reset requirement from PMA -> Need to write to TX FIFO
   // keep WR_EN asserted when dll_lock goes high until it comes out of reset completely
   // i.e. until ALL tx_digital_reset_stat in deasserted 
       wire [NUM_CHANNELS-1:0] tx_wr_en ;
    // Synchronize dll_lock and reset_stat coming from HSSI - although source clock from HSSI is not available 
    // To simplify CDC - instead of OR'ing/AND'ing all bits while crossing clock domain - generate individual wr-en per channel
    genvar ch;
    generate 
       for (ch=0; ch< NUM_CHANNELS; ch=ch+1) begin : WR_EN_GEN
           wire [NUM_CHANNELS-1:0] tx_digitalreset_stat_sync, tx_dll_lock_sync;

              alt_e40_status_sync #(
                   .WIDTH(2)
               )  stat_synchronizer_inst  (
                   .clk(tx_coreclkin[ch]),
                   .din ({tx_dll_lock[ch]     ,tx_digitalreset_stat[ch]     }),
                   .dout({tx_dll_lock_sync[ch],tx_digitalreset_stat_sync[ch]})
               );
           
           assign tx_wr_en[ch] = ( tx_dll_lock_sync[ch] & tx_digitalreset_stat_sync[ch] ) ? 1'b1 : tx_fifo_wr_en[ch]; 
       end
    endgenerate
    // --------------------------------------------------------------------------------
    // Mapping between 4x66 parallel data ports and PMA 4x64 data and 4x2 control ports
    // --------------------------------------------------------------------------------

    // TX parallel data is mapped as below -->
    // tx_parallel_data: For each 80 bit word  the 64 active data bits are tx_parallel_data[71:40,31:0];
    // tx_control: For each 80 bit word  the 2 active data bits are tx_parallel_data[33:32];
    // tx_enh_data_valid: For each 80 bit word  the 1 active data bits are tx_parallel_data[36];
    // tx_err_ins   : For each 80 bit word  the 1 active data bits are tx_parallel_data[76];
    // tx_fifo_wr_en: For each 80 bit word  the 1 active data bits are tx_parallel_data[79];
    // RX parallel data is mapped as below -->
    // rx_parallel_data: For each 80 bit word  the 64 active data bits are rx_parallel_data[71:40,31:0];
    // rx_control: For each 80 bit word  the 2 active data bits are rx_parallel_data[33:32];
    // rx_enh_data_valid: For each 80 bit word  the 1 active data bits are rx_parallel_data[36];
    // rx_data_valid: For each 80 bit word  the 1 active data bits are rx_parallel_data[79];

    (* altera_attribute = "-name FORCE_HYPER_REGISTER_FOR_PERIPHERY_CORE_TRANSFER ON" *)
    reg  [319:0] rx_data_hio;

    wire [319:0] tx_data_int;
    reg  [319:0] rx_data_int;
    reg  [319:0] tx_data_reg;
    wire [319:0] rx_data_reg;
    reg  [3:0]  rx_fifo_rd_en_reg;
    genvar i;
    generate
        for (i = 0; i < NUM_CHANNELS; i=i+1) begin : channel_loop
            assign tx_data_int[80*(i + 1)-1:80*i]   = {tx_wr_en[i],2'b0,tx_err_ins[i],4'b0,tx_parallel_data[(66*i)+63:(66*i)+32],4'b0001,2'b0,tx_parallel_data[(66*i)+65:(66*i)+64],tx_parallel_data[(66*i)+31:66*i]};
            assign rx_parallel_data[66*(i + 1)-1:66*i] = {rx_data_int[(80*i)+33:(80*i)+32],rx_data_int[(80*i)+71:(80*i)+40],rx_data_int[(80*i)+31:80*i]};
	    assign rx_data_valid[i] = rx_data_int[(i*80)+79];

           // Pipeline TX DATA for KR mode since data muxing in KR makes timing closure complicated
           if (ENABLE_KR4) begin
               always @ (posedge tx_coreclkin[i])
                 tx_data_reg[80*(i + 1)-1:80*i]  <= tx_data_int[80*(i + 1)-1:80*i]  ;
               always @ (posedge rx_coreclkin[1]) // since PCS/MAC is clocked by coreclkin[1]
                 rx_fifo_rd_en_reg[i]           <= rx_fifo_rd_en[i];	   
               always @ (posedge rx_coreclkin[i])
                 rx_data_int[80*(i + 1)-1:80*i] <= rx_data_hio[80*(i + 1)-1:80*i];    
               always @ (posedge rx_coreclkin[i])
                 rx_data_hio[80*(i + 1)-1:80*i] <= rx_data_reg[80*(i + 1)-1:80*i];    
        end
           else begin
	       always @ (*)  begin
                 tx_data_reg[80*(i + 1)-1:80*i]  = tx_data_int[80*(i + 1)-1:80*i]  ;
                 rx_data_int[80*(i + 1)-1:80*i]  = rx_data_reg[80*(i + 1)-1:80*i];    
                 rx_fifo_rd_en_reg[i]            = rx_fifo_rd_en[i];	   
        end
           end
        end
    endgenerate

    ex_40g_s10_alt_e40_181_elnlbii s10_native_phy (
         .tx_digitalreset            (tx_digitalreset),
         .rx_digitalreset            (rx_digitalreset),
         .rx_digitalreset_stat       (rx_digitalreset_stat),
         .tx_digitalreset_stat       (tx_digitalreset_stat),
        .tx_analogreset             (tx_analogreset),
        .rx_analogreset             (rx_analogreset),
        .tx_analogreset_stat        (tx_analogreset_stat),
        .rx_analogreset_stat        (rx_analogreset_stat),
        .tx_dll_lock                (tx_dll_lock),
        .tx_cal_busy                (tx_cal_busy),
        .rx_cal_busy                (rx_cal_busy),
        .tx_serial_clk0             (tx_serial_clk0),
        .rx_cdr_refclk0             (rx_cdr_refclk0),
        .tx_serial_data             (tx_serial_data),
        .rx_serial_data             (rx_serial_data),
        .rx_set_locktodata          ({4{rx_set_locktodata}}) ,
        .rx_set_locktoref           ({4{rx_set_locktoref}} ) ,
        .rx_is_lockedtoref          (rx_is_lockedtoref),
        .rx_is_lockedtodata         (rx_is_lockedtodata),
        .tx_coreclkin               (tx_coreclkin),
        .rx_coreclkin               (rx_coreclkin),
        .tx_clkout                  (tx_clkout),
        .rx_clkout                  (rx_clkout),
	.rx_pma_iqtxrx_clkout       (rx_pma_iqtxrx_clkout), // div 40 clk, 257.8125, required when RX PLL is used as FPLL	
        .tx_parallel_data           (tx_data_reg),
        .rx_parallel_data           (rx_data_reg),
        .rx_bitslip                 (rx_bitslip),
        .rx_enh_clr_errblk_count    (rx_enh_clr_errblk_count),
        .rx_enh_blk_lock            (rx_enh_blk_lock),
        .rx_fifo_rd_en              (rx_fifo_rd_en),
        .tx_fifo_full               (tx_fifo_full),
        .tx_fifo_empty              (tx_fifo_empty),
        .tx_fifo_pfull              (tx_fifo_pfull),
        .tx_fifo_pempty             (tx_fifo_pempty),
        .rx_fifo_full               (rx_fifo_full),
        .rx_fifo_empty              (rx_fifo_empty),
        .rx_fifo_pfull              (rx_fifo_pfull),
        .rx_fifo_pempty             (rx_fifo_pempty),
        .reconfig_clk               (reconfig_clk),
        .reconfig_reset             (reconfig_reset),
        .reconfig_write             (reconfig_write),
        .reconfig_read              (reconfig_read),
        .reconfig_address           (reconfig_address),
        .reconfig_writedata         (reconfig_writedata),
        .reconfig_readdata          (reconfig_readdata),
        .reconfig_waitrequest       (reconfig_waitrequest),
        .rx_seriallpbken            (rx_seriallpbken)
					   
    );

endmodule

