// (C) 2001-2018 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


// =================================================================================
// Creates a pulsed signal from an input signal
// =================================================================================

module pulse_gen (
    input  logic clk,            // the DPS clock
    input  logic rst,            // active high reset
    input  logic signal,         // the user's pulsed_signal signal
    output logic pulsed_signal  // the pulsed_signal signal for the IOPLL atom
 );
 
    //-------------------------------------------------------------------------
    // States
    localparam IDLE        = 0,  // Idle state: signal = 0, pulsed_signal = 0
               PULSE       = 1,  // Activate state: pulsed_signal = 1
               WAIT        = 2;  // Wait for signal to go low

    //-------------------------------------------------------------------------
    // FSM current and next states
    logic [1:0] state, next;     
    
    // State update
    always @(posedge clk) begin
        if (rst)    state <= IDLE;
        else        state <= next; 
    end  

    //-------------------------------------------------------------------------    
    // Next-state and output logic
    always @(*) begin
        next          = IDLE;  // Default next state 
        pulsed_signal = 1'b0;  // Default output
        
        case (state)
            IDLE :  begin
                        if (signal)  next = PULSE;
                        else         next = IDLE;
                    end     
                         
            PULSE : begin
                        pulsed_signal = 1'b1;
                        next          = WAIT;
                    end
                         
            WAIT :  begin         
                        if (~signal) next = IDLE;
                        else         next = WAIT;                  
                    end  
        endcase
    end
     
 endmodule




