// ============================================================================
// Copyright (c) 2019 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Wed Jun 26 16:32:24 2019
// ============================================================================

//`define ENABLE_DDR4A
//`define ENABLE_DDR4B
//`define ENABLE_FMC
//`define ENABLE_FMCP
//`define ENABLE_PCIE
//`define ENABLE_QSFP28A
//`define ENABLE_QSFP28B
//`define ENABLE_HPS

module S10C_top(

      ///////// CLOCK /////////
      input              CLK_100_B3I,
      input              CLK_50_B2F,
      input              CLK_50_B2L,
      input              CLK_50_B3C,
      input              CLK_50_B3I,
      input              LVDS_REFCLK_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON,

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 1: 0]   LED, //LED is Low-Active

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [17: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output             DDR4A_CK,
      output             DDR4A_CK_n,
      output   [ 1: 0]   DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output   [ 1: 0]   DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output   [ 1: 0]   DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [17: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output             DDR4B_CK,
      output             DDR4B_CK_n,
      output   [ 1: 0]   DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [ 1: 0]   DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [ 1: 0]   DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

      ///////// SI5341A /////////
      inout              SI5341A_I2C_SCL,
      inout              SI5341A_I2C_SDA,
      output             SI5341A_OE_n,
      output             SI5341A_RST_n,

`ifdef ENABLE_FMC
      ///////// FMC /////////
      input    [ 1: 0]   S10C_KEY,
      input    [ 1: 0]   S10C_SW,
      output   [ 3: 0]   S10C_LED,
      inout    [35: 0]   FMC_GPIO_D,
`endif /*ENABLE_FMC*/

`ifdef ENABLE_FMCP
      ///////// FMCP /////////
      input              FMCP_REFCLK_p,
      inout              Si5340A_I2C_SCL,                    // FMCP_SCL
      inout              Si5340A_I2C_SDA,                    // FMCP_SDA
      output             SI5340A_OE_n,
      output             SI5340A_RST_n,

`ifdef ENABLE_PCIE        
      input              DG_PCIe_RST_n,
      output             DG_PEWAKE_n_FMCP,
      input              DG_PM_S0_EN_FMCP,
      input              DG_PM_S3_EN_FMCP,
      input              PCIE_REFCLK_p,
      input    [ 3: 0]   PCIE_RX_p,
      output   [ 3: 0]   PCIE_TX_p,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFP28A
      input              QSFP28A_INTERRUPT_n,
      output             QSFP28A_LP_MODE,
      input              QSFP28A_MOD_PRS_n,
      output             QSFP28A_MOD_SEL_n,
      output             QSFP28A_RST_n,
      inout              QSFP28A_SCL,
      inout              QSFP28A_SDA,
      input              QSFP28A_REFCLK_p,
      input    [ 3: 0]   QSFP28A_RX_p,
      output   [ 3: 0]   QSFP28A_TX_p,
`endif /*ENABLE_QSFP28A*/

`ifdef ENABLE_QSFP28B
      input              QSFP28B_INTERRUPT_n,
      output             QSFP28B_LP_MODE,
      input              QSFP28B_MOD_PRS_n,
      output             QSFP28B_MOD_SEL_n,
      output             QSFP28B_RST_n,
      inout              QSFP28B_SCL,
      inout              QSFP28B_SDA,
      input              QSFP28B_REFCLK_p,
      input    [ 3: 0]   QSFP28B_RX_p,
      output   [ 3: 0]   QSFP28B_TX_p,
`endif /*ENABLE_QSFP28B*/
`endif /*ENABLE_FMCP*/

`ifdef ENABLE_HPS
      ///////// HPS /////////
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      inout    [ 3: 0]   HPS_GPIO,
      inout              HPS_I2C_SCL,
      inout              HPS_I2C_SDA,
      inout              HPS_KEY,
      inout              HPS_LED,
      input              HPS_OSC_CLK,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
      input              HPS_USB_CLK,
      inout    [ 7: 0]   HPS_USB_DATA,
      input              HPS_USB_DIR,
      input              HPS_USB_NXT,
      output             HPS_USB_STP,
`endif /*ENABLE_HPS*/

      ///////// S10MAX10 /////////
      output             INFO_SPI_SCLK, 
      output             INFO_SPI_CS_n, 
      output             INFO_SPI_MOSI, 
      input              INFO_SPI_MISO 


);


//=======================================================
//  REG/WIRE declarations
//=======================================================




//=======================================================
//  Structural coding
//=======================================================




endmodule
